package yaml

import (
	"testing"

	"git.perx.ru/perxis/perxis-go/yaml/testdata"
	"github.com/stretchr/testify/assert"
	"github.com/stretchr/testify/require"
	"gopkg.in/yaml.v3"
)

func TestTagProcessor(t *testing.T) {
	t.Run("!file", func(t *testing.T) {
		file, err := testdata.FS.Open("file/file_simple.yaml")
		require.NoError(t, err)
		defer file.Close()

		var result any
		decoder := yaml.NewDecoder(file)
		err = decoder.Decode(WithTagProcessor(testdata.FS)(&result))
		require.NoError(t, err)
		assert.Equal(t, map[string]any{"config": `server {
    listen 80;
    server_name example.com;

    location / {
        root /var/www/example.com/html;
        index index.html index.htm;
        try_files $uri $uri/ =404;
    }
}`}, result)
	})
	t.Run("!include", func(t *testing.T) {
		t.Run("simple", func(t *testing.T) {
			file, err := testdata.FS.Open("include/include_simple.yaml")
			require.NoError(t, err)
			defer file.Close()

			var result any
			decoder := yaml.NewDecoder(file)
			err = decoder.Decode(WithTagProcessor(testdata.FS)(&result))
			require.NoError(t, err)
			assert.Equal(t, map[string]any{"data": map[string]any{"text": "Hello, World!"}}, result)
		})
		t.Run("with merge", func(t *testing.T) {
			file, err := testdata.FS.Open("include/include_merge.yaml")
			require.NoError(t, err)
			defer file.Close()

			var result any
			decoder := yaml.NewDecoder(file)
			err = decoder.Decode(WithTagProcessor(testdata.FS)(&result))
			require.NoError(t, err)
			assert.Equal(t, map[string]any{
				"server": map[string]any{
					"host": "dev.perx.ru",
					"port": 3000,
					"ssl":  true,
					"tags": []any{"http", "api"},
				},
			}, result)
		})
	})
}
